#include <cstdlib>
#include <fstream>
#include <iostream>

#include "gcsa.h"
#include "graph.h"

#include <misc/utils.h>


using namespace CSA;


int
main(int argc, char** argv)
{
  std::cout << "GCSA builder" << std::endl;
  std::cout << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: build_index base_name" << std::endl;
    return 1;
  }

  std::string base_name(argv[1]);
  std::cout << "Input:  " << base_name << std::endl;
  std::cout << std::endl;

  double start = readTimer();

  Graph graph(base_name);
  if(!graph.ok) { return 2; }
  graph.printInfo();

  PathGraph* pg = new PathGraph(graph);
  pg->printInfo();
  while(pg->status != PathGraph::sorted)
  {
    if(pg->status != PathGraph::ok)
    {
      std::cerr << "Error: Invalid PathGraph!" << std::endl;
      delete pg; pg = 0;
      return 3;
    }
    PathGraph* next = new PathGraph(*pg);
    delete pg; pg = next;
    pg->printInfo();
  }
  std::cout << std::endl;

  GCSA gcsa(*pg, true);
  delete pg; pg = 0;
  gcsa.writeTo(base_name);

  double time = readTimer() - start;
  std::cout << "Used " << time << " seconds." << std::endl;
  std::cout << "Memory: " << memoryUsage() << " kB" << std::endl;
  std::cout << std::endl;

  return 0;
}
